﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Multihalmaz_típus
{
    using Elemtípus = Int32;//Halmaz elemeinek típusa
    class MultiHalmaz
    {
        private const Elemtípus minelem = -50; //Min'Elemtípus
        private const Elemtípus maxelem = 50; //Max'Elemtípus
        private int N = 0;//Elemszám
        private int[] elemek = new int[maxelem-minelem+1];//A halmaz elemeit tartalmazó tömb

        public void Beolvasás()
        {
            Console.Error.WriteLine("A multihalmaz elemszáma: ");
            N = Convert.ToInt32(Console.ReadLine());
            Üres();
            for (int i = 1; i <= N; i++)
            {
                Console.Error.WriteLine("Írd be a(z) " + i + ". elemet és darabszámát (elem db)!");
                string sor = Console.ReadLine();
                string[] adatok = sor.Split(' ');
                Elemtípus e = Convert.ToInt32(adatok[0]);
                int db = Convert.ToInt32(adatok[1]);
                elemek[e - minelem] = db;//Indexeltolás
            }
        }

        public void Kiírás()
        {
            Console.WriteLine("A halmaz elemei és darabszámuk: ");
            for (Elemtípus i = minelem; i <= maxelem; i++)
            {
                if (elemek[i - minelem] > 0)
                {
                    Console.WriteLine(i + " " + elemek[i - minelem]);
                }
            }
            Console.WriteLine();
        }

        public void Üres()
        {
            for (Elemtípus i = minelem; i <= maxelem; i++)
            {
                elemek[i - minelem] = 0;
            }
        }

        public bool Üres_e()
        {
            Elemtípus i = minelem;
            while(i <= maxelem && !Eleme(i))
            {
                i++;
            }
            return i > maxelem;
        }

        public void MultiHalmazba(Elemtípus e)
        {
            elemek[e - minelem]++;
        }

        public void MultiHalmazból(Elemtípus e)
        {
            if (Eleme(e))
            {
                elemek[e - minelem]--;
            }
        }

        public bool Eleme(Elemtípus e)
        {
            return elemek[e - minelem] > 0;
        }

        public int Multiplicitás(Elemtípus e)//e darabszáma
        {
            return elemek[e - minelem];
        }

        public bool Része(MultiHalmaz másik)//Az "aktuális" halmaz része-e a másiknak
        {
            Elemtípus i = minelem;
            while (i <= maxelem && elemek[i - minelem] <= másik.elemek[i - minelem])
            {
                i++;
            }
            return i > maxelem;
        }

        private Elemtípus Min(Elemtípus a, Elemtípus b)
        {
            if (a < b)
            {
                return a;
            }
            else
            {
                return b;
            }
        }

        private Elemtípus Max(Elemtípus a, Elemtípus b)
        {
            if (a > b)
            {
                return a;
            }
            else
            {
                return b;
            }
        }

        public MultiHalmaz Unió(MultiHalmaz másik)
        {
            MultiHalmaz unióhalmaz = new MultiHalmaz();
            for (Elemtípus i = minelem; i <= maxelem; i++)
            {
                unióhalmaz.elemek[i - minelem] = elemek[i - minelem] + másik.elemek[i - minelem];
            }
            return unióhalmaz;
        }

        public MultiHalmaz Metszet(MultiHalmaz másik)
        {
            MultiHalmaz metszethalmaz = new MultiHalmaz();
            for (Elemtípus i = minelem; i <= maxelem; i++)
            {
                metszethalmaz.elemek[i - minelem] = Min(elemek[i - minelem], másik.elemek[i - minelem]);
            }
            return metszethalmaz;
        }

        public MultiHalmaz Maximum(MultiHalmaz másik)
        {
            MultiHalmaz maximumhalmaz = new MultiHalmaz();
            for (Elemtípus i = minelem; i <= maxelem; i++)
            {
                maximumhalmaz.elemek[i - minelem] = Max(elemek[i - minelem], másik.elemek[i - minelem]);
            }
            return maximumhalmaz;
        }

        public MultiHalmaz Különbség(MultiHalmaz másik)
        {
            MultiHalmaz különbséghalmaz = new MultiHalmaz();
            for (Elemtípus i = minelem; i <= maxelem; i++)
            {
                különbséghalmaz.elemek[i - minelem] = Max(elemek[i - minelem] - másik.elemek[i - minelem], 0);
            }
            return különbséghalmaz;
        }

        //Unió, Metszet, Maximum, Különbség operátorokkal
        public static MultiHalmaz operator +(MultiHalmaz a, MultiHalmaz b)
        {
            MultiHalmaz unióhalmaz = new MultiHalmaz();
            for (Elemtípus i = minelem; i <= maxelem; i++)
            {
                unióhalmaz.elemek[i - minelem] = a.elemek[i - minelem] + b.elemek[i - minelem];
            }
            return unióhalmaz;
        }

        public static MultiHalmaz operator *(MultiHalmaz a, MultiHalmaz b)
        {
            MultiHalmaz metszethalmaz = new MultiHalmaz();
            for (Elemtípus i = minelem; i <= maxelem; i++)
            {
                metszethalmaz.elemek[i - minelem] = metszethalmaz.Min(a.elemek[i - minelem], b.elemek[i - minelem]);
            }
            return metszethalmaz;
        }

        public static MultiHalmaz operator /(MultiHalmaz a, MultiHalmaz b)
        {
            MultiHalmaz maximumhalmaz = new MultiHalmaz();
            for (Elemtípus i = minelem; i <= maxelem; i++)
            {
                maximumhalmaz.elemek[i - minelem] = maximumhalmaz.Max(a.elemek[i - minelem], b.elemek[i - minelem]);
            }
            return maximumhalmaz;
        }

        public static MultiHalmaz operator -(MultiHalmaz a, MultiHalmaz b)
        {
            MultiHalmaz különbséghalmaz = new MultiHalmaz();
            for (Elemtípus i = minelem; i <= maxelem; i++)
            {
                különbséghalmaz.elemek[i - minelem] = különbséghalmaz.Max(a.elemek[i - minelem] - b.elemek[i - minelem], 0);
            }
            return különbséghalmaz;
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            MultiHalmaz h1 = new MultiHalmaz();
            h1.Beolvasás();
            if (h1.Üres_e())
            {
                Console.WriteLine("A h1 halmaz üres");
            }
            else
            {
                Console.WriteLine("A h1 halmaz nem üres");
            }
            h1.Kiírás();
            if (h1.Eleme(2))
            {
                Console.WriteLine("A 2 eleme a halmaznak, így ki tudjuk venni belőle.");
                h1.MultiHalmazból(2);
            }
            else
            {
                Console.WriteLine("A 2 nem eleme a halmaznak, így nem történt semmi.");
            }
            h1.Kiírás();
            MultiHalmaz h2 = new MultiHalmaz();
            if (h2.Üres_e())
            {
                Console.WriteLine("A h2 halmaz üres");
            }
            else
            {
                Console.WriteLine("A h2 halmaz nem üres");
            }
            h2.Beolvasás();
            h2.Kiírás();
            //h1.Unió(h2).Kiírás();
            (h1 + h2).Kiírás();
            //h1.Metszet(h2).Kiírás();
            (h1 * h2).Kiírás();
            //h1.Maximum(h2).Kiírás();
            (h1 / h2).Kiírás();
            //h1.Különbség(h2).Kiírás();
            (h1 - h2).Kiírás();
            if (h2.Része(h1))
            {
                Console.WriteLine("h2 része h1-nek.");
            }
            else
            {
                Console.WriteLine("h2 nem része h1-nek.");
            }
            h1.Kiírás();
            h2.Kiírás();
        }
    }
}
